@@ -13,6 +13,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
              && (conf.level < 1)))
             stop("conf.level must be a single number between 0 and 1")
     }
+    MIDP <- NULL
 
     if(!is.null(y)) {
         DNAME <- paste(deparse(substitute(x)), "and",
@@ -87,14 +88,14 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                                qu <- qsignrank(alpha, n)
                                if(qu == 0) qu <- 1
                                uci <- diffs[qu]
-                               c(uci, NA)
+                               c(uci, Inf)
                            },
                            "less"= {
                                qu <- qsignrank(alpha, n)
                                if(qu == 0) qu <- 1
                                ql <- n*(n+1)/2 - qu
                                lci <- diffs[ql+1]
-                               c(NA, lci)        
+                               c(-Inf, lci)        
                            })
                 attr(cint, "conf.level") <- conf.level    
             }
@@ -135,13 +136,13 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                                    qu <- qperm(alpha, r, n) 
                                    if (qu <= min(w)) uci <- min(diffs)
                                    else uci <- max(diffs[w <= qu])
-                                   c(uci, NA)
+                                   c(uci, Inf)
                                },
                                "less"= {
                                    ql <- qperm(1-alpha, r, n)
                                    if (ql >= max(w)) lci <- max(diffs)
                                    else lci <- min(diffs[w > ql])
-                                   c(NA, lci)
+                                   c(-Inf, lci)
                          })
                      attr(cint, "conf.level") <- conf.level    
                 }
@@ -175,11 +176,12 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                     ## These are sample based limits for the median
                     mumin <- min(x)
                     mumax <- max(x)
-                    ## wdiff(d, zq) returns the abolute difference between
+                    ## wdiff(d, zq) returns the absolute difference between  
                     ## the asymptotic Wilcoxon statistic of x - mu - d and
-                    ## the quantile zq 
+                    ## the quantile zq.
+                    CORRECTION.CI <- 0
                     wdiff <- function(d, zq) {
-                        xd <- x  - d
+                        xd <- x - d
                         xd <- xd[xd != 0]
                         nx <- length(xd)
                         dr <- rank(abs(xd))
@@ -187,7 +189,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                         NTIES.CI <- table(dr)
                         zd <- zd - nx * (nx + 1)/4
                         SIGMA.CI <- sqrt(nx * (nx + 1) * (2 * nx + 1) / 24
-                                         - sum(NTIES.CI^3 -  NTIES.CI) / 48)
+                                     - sum(NTIES.CI^3 -  NTIES.CI) / 48)
                         if(correct) {
                             CORRECTION.CI <-
                                 switch(alternative,
@@ -196,9 +198,9 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                                        "less" = -0.5)
                         }
                         zd <- (zd - CORRECTION.CI) / SIGMA.CI
-                        abs(zd - zq)
+                        zd - zq
                     }
-                    ## Here we optimize the function wdiff in d over the set
+                    ## Here we search the root of the function wdiff on the set
                     ## c(mumin, mumax).
                     ##
                     ## This returns a value from c(mumin, mumax) for which
@@ -209,22 +211,22 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                     ##
                     ## As in the exact case, interchange quantiles.
                     cint <- switch(alternative, "two.sided" = {
-                        u <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                      zq=qnorm(1-alpha/2))$minimum
-                        l <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                      zq=qnorm(alpha/2))$minimum
-                        c(u, l)
+                        l <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                     zq=qnorm(alpha/2, lower=FALSE))$root
+                        u <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                     zq=qnorm(alpha/2))$root
+                        c(l, u)
                     }, "greater"= {
-                        u <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                      zq=qnorm(1-alpha))$minimum
-                        c(u, NA)
+                        l <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                     zq=qnorm(alpha, lower=FALSE))$root
+                        c(l, +Inf)
                     }, "less"= {
-                        l <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                      zq=qnorm(alpha))$minimum
-                        c(NA, l)
+                        u <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                     zq=qnorm(alpha))$root
+                        c(-Inf, u)
                     })
                     attr(cint, "conf.level") <- conf.level    
-                }
+                 }
 
 #                if(exact && TIES) {
 #                warning("Cannot compute exact p-value with ties")
@@ -289,14 +291,14 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                                    qu <- qwilcox(alpha, n.x, n.y)
                                    if(qu == 0) qu <- 1
                                    uci <- diffs[qu]
-                                   c(uci, NA)
+                                   c(uci, Inf)
                                },
                                "less"= {
                                    qu <- qwilcox(alpha, n.x, n.y)
                                    if(qu == 0 ) qu <- 1
                                    ql <- n.x*n.y - qu
                                    lci <- diffs[ql + 1]
-                                   c(NA, lci)
+                                   c(-Inf, lci)
                                })
                     attr(cint, "conf.level") <- conf.level    
                 }
@@ -313,7 +315,7 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                     ## mean(y) - mean(x) in the two-sample case (cf. the
                     ## one-sample case).
                     alpha <- 1 - conf.level
-                    diffs <- sort(outer(y, x, "-"))
+                    diffs <- sort(outer(x, y, "-"))
                     w <- cumsum(rep(1, n.x*n.y))
                     dup <- which(duplicated(diffs))
                     indx <- 1:length(diffs)
@@ -335,13 +337,13 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
 	                           qu <- qperm(alpha, r, n.x) - n.x*(n.x+1)/2
                                    if (qu <= min(w)) uci <- min(diffs)
                                    else uci <- max(diffs[w <= qu])
-                                   c(uci, NA)
+                                   c(uci, +Inf)
                                },
                                "less"= {
                                    ql <- qperm(1-alpha, r, n.x) - n.x*(n.x+1)/2
                                    if (ql >= max(w)) lci <- max(diffs)
                                    else lci <- min(diffs[w > ql]) 
-                                   c(NA, lci)
+                                   c(-Inf, lci)
                               })
                      attr(cint, "conf.level") <- conf.level    
                 }
@@ -368,16 +370,17 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
 
             if(conf.int) {
                 ## Asymptotic confidence interval for the location
-                ## parameter mean(y) - mean(x) in the two-sample case
+                ## parameter mean(x) - mean(y) in the two-sample case
                 ## (cf. one-sample case).
                 ##
                 ## Algorithm not published, for a documentation see the
-                ## one sample case.
+                ## one-sample case.
                 alpha <- 1 - conf.level
-                mumin <- min(y) - max(x)
-                mumax <- max(y) - min(x)
+                mumin <- min(x) - max(y)
+                mumax <- max(x) - min(y)
+                CORRECTION.CI <- 0
                 wdiff <- function(d, zq) {
-                    dr <- rank(c(x - mu, y - d)) 
+                    dr <- rank(c(x - d, y))
                     NTIES.CI <- table(dr)
                     dz <- (sum(dr[seq(along = x)])
                            - n.x * (n.x + 1) / 2 - n.x * n.y / 2)
@@ -393,22 +396,22 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
                                       - sum(NTIES.CI^3 - NTIES.CI)
                                       / ((n.x + n.y) * (n.x + n.y - 1))))
                     dz <- (dz - CORRECTION.CI) / SIGMA.CI
-                    abs(dz - zq)
+                    dz - zq
                 }
                 cint <- switch(alternative, "two.sided" = {
-                    u <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                  zq=qnorm(alpha/2))$minimum
-                    l <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                  zq=qnorm(1 - alpha/2))$minimum
-                    c(u, l)
+                    l <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                  zq=qnorm(alpha/2, lower=FALSE))$root
+                    u <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                  zq=qnorm(alpha/2))$root
+                    c(l, u)
                 }, "greater"= {
-                    u <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                  zq=qnorm(alpha))$minimum
-                    c(u, NA)
+                    l <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                  zq=qnorm(alpha, lower=FALSE))$root
+                    c(l, +Inf)
                 }, "less"= {
-                    l <- optimize(wdiff, c(mumin, mumax), tol=1e-4,
-                                  zq=qnorm(1 - alpha))$minimum
-                    c(NA, l)
+                    u <- uniroot(wdiff, c(mumin, mumax), tol=1e-4,
+                                  zq=qnorm(alpha))$root
+                    c(-Inf, u)
                 })
                 attr(cint, "conf.level") <- conf.level    
             }
@@ -422,15 +425,23 @@ function(x, y = NULL, alternative = c("two.sided", "less", "greater"),
         }
     }
 
-    if (!is.null(MIDP)) names(MIDP) <- "point prob"
-
-    RVAL <- list(statistic = STATISTIC,
-                 parameter = MIDP,
-                 p.value = PVAL, 
-                 null.value = c(mu = mu),
-                 alternative = alternative,
-                 method = METHOD, 
-                 data.name = DNAME)
+    if (!is.null(MIDP)) {
+        names(MIDP) <- "point prob"
+        RVAL <- list(statistic = STATISTIC,
+                     parameter = MIDP,
+                     p.value = PVAL, 
+                     null.value = c(mu = mu),
+                     alternative = alternative,
+                     method = METHOD, 
+                     data.name = DNAME)
+    } else {
+        RVAL <- list(statistic = STATISTIC,
+                     p.value = PVAL, 
+                     null.value = c(mu = mu),  
+                     alternative = alternative,
+                     method = METHOD, 
+                     data.name = DNAME)
+    }
     if(conf.int)
         RVAL$conf.int <- cint
     class(RVAL) <- "htest"
